Package: MVar
Type: Package
Title: Multivariate Analysis
Version: 2.2.9
Date: 2026-05-07
Imports: graphics, grDevices, MASS, stats
Authors@R: c(
           person(given = c("Paulo Cesar"), family = "Ossani", 
             email = "ossanipc@hotmail.com", 
             role = c("aut", "cre"), 
             comment = c(ORCID = "0000-0002-6617-8085")), 
           person(given = c("Marcelo Angelo"), family = "Cirillo", 
             email = "macufla@gmail.com", 
             role = c("aut"), 
             comment = c(ORCID = "0000-0002-2647-439X")))
Description: Multivariate analysis, having functions that perform simple correspondence analysis (CA) and multiple correspondence analysis (MCA), principal components analysis (PCA), canonical correlation analysis (CCA), factorial analysis (FA), multidimensional scaling (MDS), linear (LDA) and quadratic discriminant analysis (QDA), hierarchical and non-hierarchical cluster analysis, simple and multiple linear regression, multiple factor analysis (MFA) for quantitative, qualitative, frequency (MFACT) and mixed data, biplot, scatter plot, projection pursuit (PP), grant tour method and other useful functions for the multivariate analysis.
License: GPL-3
Encoding: UTF-8
LazyData: true
NeedsCompilation: yes
Author: Paulo Cesar Ossani [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6617-8085>),
  Marcelo Angelo Cirillo [aut] (ORCID:
    <https://orcid.org/0000-0002-2647-439X>)
Maintainer: Paulo Cesar Ossani <ossanipc@hotmail.com>
Repository: CRAN
Packaged: 2026-05-07 12:46:16 UTC; sec-l
Date/Publication: 2026-05-07 13:52:08 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-07 16:10:58 UTC; unix
Archs: MVar.so.dSYM
