exportPattern("^[^\\.]")
importFrom("graphics", "abline", "arrows", "barplot", "legend", "par", "plot", 
           "points", "segments", "strheight", "strwidth", "symbols", "text", 
           "hist", "lines", "pairs", "grid", "rect", "axis", "box")
importFrom("stats", "cor", "cov", "var", "qchisq", "runif", "sd", "pchisq", "pf", "qf",
           "varimax", "cmdscale", "dist", "integrate", "prcomp", "rnorm", "as.dendrogram",
           "as.dist", "cutree", "hclust", "kmeans", "rect.hclust", "Box.test", 
           "bartlett.test", "pt", "qqline","qqnorm", "qt", "lm", "predict", "promax")
importFrom("MASS", "ginv", "lda", "qda")
importFrom("grDevices","dev.off","png", "dev.new", "xy.coords", "chull")
useDynLib(MVar, .registration = TRUE)
